	list	p=16f877A	; list directive to define processor
	include	<p16F877A.inc>
    
	errorlevel -302,-207	; suppress message 302 from list file
	errorlevel -305,-207	; suppress message 302 from list file

	__config _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_OFF & _CPD_OFF

;===========================================================================  
;
;     PIC16F877-A   
;                                 __________  
;                 +5v------!MCLR |1       40| RB7-I---- (KEYPAD  COL4)  
;            (Unused)------  RA0 |2       39| RB6-I---- (KEYPAD  COL3)
;            (Unused)------  RA1 |3       38| RB5-I---- (KEYPAD  COL2)
;            (Unused)------  RA2 |4       37| RB4-I---- (KEYPAD  COL1)
;            (Unused)------  RA3 |5       36| RB3  ---- (Unused)  
;            (Unused)------  RA4 |6       35| RB2  ---- (Unused)  
;            (Unused)------  RA5 |7       34| RB1  ---- (Unused)                  
;            (Unused)------  RE0 |8       33| RB0/INT-- (Unused)  
;            (Unused)------  RE1 |9       32| VDD  ---- +5v  
;            (Unused)------  RE2 |10      31| VSS  ---- GND  
;                 +5v------  VDD |11      30| RD7-O---- (KEYPAD  ROW4)   
;                 GND------  VSS |12      29| RD6-O---- (KEYPAD  ROW3)
;                XTAL------ OSC1 |13      28| RD5-O---- (KEYPAD  ROW2)  
;                XTAL------ OSC2 |14      27| RD4-O---- (KEYPAD  ROW1) 
;             (Unused)-----  RC0 |15      26| RC7-O---- (LCD pin 14)  LCD_DB7    
; LCD_rs (LCD pin 4) ------O-RC1 |16      25| RC6-O---- (LCD pin 13)  LCD_DB6  
; LCD_rw (LCD pin 5) ------O-RC2 |17      24| RC5-O---- (LCD pin 12)  LCD_DB5  
; LCD_e  (LCD pin 6) ------O-RC3 |18      23| RC4-O---- (LCD pin 11)  LCD_DB4  
;            (Unused)------  RD0 |19      22| RD3  ---- (Unused)  
;            (Unused)------  RD1 |20      21| RD2  ---- (Unused)  
;                                 ----------  
;
;	LCD (Hitachi)  
;                                 ________  
;    		                 VSS |1       |   
;                            VDD |2       |   
;                 CONTRAST   VEE |3       |   
;                 RC1------  RS  |4       |   
;                 RC2------  R/W |5       |   
;                 RC3------  E   |6       |   
;                       (Unused) |7       |                   
;                       (Unused) |8       |   
;                       (Unused) |9       |   
;                       (Unused) |10      |  
;                 RC4------  DB4 |11      |    
;                 RC5------  DB5 |12      |   
;                 RC6------  DB6 |13      |      
;                 RC7------  DB7 |14      |     
;          BACKLIGHT POWER   VDD |15      |     
;                   GROUND   VSS |16      |   
;                                 --------  
;
;
;    KEYPAD
;                     RB4        RB5         RB6          RB7
;                      |          |           |            |
;                      |          |           |            |
;                      |          |           |            |
;
;                    COL 1       COL 2       COL 3        COL 4   
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 1 |    1    |  |   2    |  |   3    |   |   A    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 2 |    4    |  |   5    |  |   6    |   |   B    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 3 |    7    |  |   8    |  |   9    |   |   C    |
;                 +---------+  +--------+  +--------+   +--------+  
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 4 |    *    |  |   0    |  |   #    |   |   D    |
;                 +---------+  +--------+  +--------+   +--------+  
;
;
;=============================================================================
; DECLARATIONS
;=============================================================================
RESET_V	EQU	0X0000			; ADDRESS OF RESET VECTOR.
ISR_V	EQU	0X0004			; ADDRESS OF INTERRUPT VECTOR.
OSC_FREQ EQU D'4000000'		; OSCILLATOR FREQUENCY IS 4 MHZ.

LCD_RS	equ	1				;0 = COMMANDS. 1 = DATA.
LCD_RW	equ	2				;0 = WRITE TO LCD. 1 = READ LCD.
LCD_E	equ	3				;ENABLE THE LCD TO ACCEPT INPUT. PULSE THIS.
LCDBUSY	equ	7				;WHEN THE LCD IS BUSY RC7 IS SET HIGH.


; PORTB BITS
RB7		equ	7				;KEYBOARD, COL 4.
RB6		equ	6				;KEYBOARD, COL 3.
RB5		equ	5 				;KEYBOARD, COL 2.
RB4		equ	4				;KEYBOARD, COL 1.

; PORTD BITS
RD7		equ	7				;KEYBOARD, ROW 1.
RD6		equ	6				;KEYBOARD, ROW 2.
RD5		equ	5				;KEYBOARD, ROW 3.
RD4		equ	4				;KEYBOARD, ROW 4.

RD3		equ	3				;KEY CODE, BIT 3, USE 1K + LED TO VSS.
RD2		equ	2				;KEY CODE, BIT 2, USE 1K + LED TO VSS.
RD1		equ	1				;KEY CODE, BIT 1, USE 1K + LED TO VSS.
RD0		equ	0				;KEY CODE, BIT 0, USE 1K + LED TO VSS.

KEYS	equ	0				;BIT FOR SETTING THE KEYBOARD FLAG
		

; REGISTERS.
		cblock  H'20'       ; NOTE: START DATA BLOCK.
;FREQUENCY VALUES.
POS_10M	
POS_1M
POS_100K
POS_10K
POS_1K
POS_100
POS_10
POS_1

FCHAR_MAX
FCHAR_CNT
FCHAR_STRT				;START OF POSTION_1 IN ADDRESS LOCATION...
FLAGS

;KEYPAD VARIABLES.       		
TEMP					;TEMP STORAGE.
LAST_KEY				;LAST KEY PRESSED.
LAST_CHAR				;LAST KEY PRESSED.

		
TIMER1						; USED IN DELAY ROUTINES
TIMER2						;   " 

;LCD VARIABLES.
LCD_CHAR					;THE BYTE REPRESENTING THE LCD CHARACTER. 
LCD_READ 					;VARIABLE CONTAINING THE READ FOR LCD_BUSY. 
LCD_VALUE 					;THE LCD RS LINE - FLAG VALUE. 

		
; MISC.
DELAY_VAL					; USED IN DELAY ROUTINES.
X_DLY_VAL					; "
COUNT						; USED AS G.P. COUNTER.


		
_W         					; TEMPORARY STORE FOR W.
_STATUS        				; TEMPORARY STORE FOR STATUS.
_PCLATH		
		endc				; NOTE: END OF DATA BLOCK.    
   
;=============================================================================
; PROGRAM START.
;***************
;
; NOTE: - 	Programs always start at ORG 0x00. The reset vector always starts
;			at 0x04 so four lines down from 00. This means if you are 
;			including an interupt routine you must jump over 0x04 or your 
;			will run into it.		
;=============================================================================
		org		RESET_V		; RESET vector location
		reset	GOTO MAIN	;JUMPING TO AVOID THE INTERRUPT VECTOR.

;=============================================================================
;THIS IS THE INTERUPT ROUTINE.
;=============================================================================
INTERRUPT	org	ISR_V		;INTERRUPT VECTOR LOCATION.

		movwf	_W
		swapf	STATUS,	W
		movwf	_STATUS
		movf	PCLATH,	W	; move pclath register into w register
		movwf	_PCLATH		; save off contents of PCLATH register

		bcf		STATUS,	RP0	;SELECT BANK 0.
          	
		;call 	WAIT_15MS
		
		movlw   B'11110000'	;CHECK FOR KEY DOWN.
		movwF   TEMP  
		movf    PORTB,	W
		subwf   TEMP,	F
		btfsc	STATUS, Z
		goto    DONE		;ALL KEYS UP SO RETURN.
		
		bsf	FLAGS, KEYS
		
		;call	SHIFT_KEYS
	
DONE
		clrf    PORTB		;CLEAR PORT B.
		clrf    PORTD		;CLEAR PORT D.
		movf    PORTB,	F

 		bcf     INTCON,RBIF;CLEAR Port B interrupt FLAG

		movf	_PCLATH,W	; retrieve copy of PCLATH register
		movwf	PCLATH		; restore pre-isr PCLATH register contents
		movf    _STATUS,W	; retrieve copy of STATUS register	
		movwf   STATUS
		swapf   _W,F
		swapf   _W,W
		;bcf     INTCON,RBIF;CLEAR Port B interrupt FLAG	
		retfie

;=============================================================================
;INCLUDE FILES HERE.
;*******************
;=============================================================================

		include "KEYPAD_ROUTINES.INC"
		include "LCD_ROUTINES.INC"		
;=============================================================================
;"MAIN" STARTS HERE.
;*******************
;=============================================================================	
MAIN
		call	PIC_INIT	;INITIATE THE PIC.
		;call	LCD_INIT	;INITIATE THE LCD NEXT.
	;	call	FLASH4		;FLASH ALL FOUR LEDS MOMENTARILY.
		;call	OPEN_MESG	;DISPLAY OPENING MESSAGE.
		
		clrf	FCHAR_MAX
		
		movlw	H'27'
		movwf	FCHAR_STRT

		movlw	H'00'
		movwf	FCHAR_MAX
		
		movlw	H'02'
		movwf	LAST_CHAR
		
		movlw	H'00'
		movwf	FLAGS	
	
LOOP
		nop
		btfsc	FLAGS,KEYS
		call	SHIFT_KEYS
		goto 	LOOP


;=============================================================================
;INITIATE THE PIC MICROCONTROLLER.
;=============================================================================		
PIC_INIT
		clrf	STATUS		; DO INITIALIZATION, SELECT BANK 0
		clrf	INTCON		; CLEAR INT-FLAGS, DISABLE INTERRUPTS
		clrf	PCLATH		; KEEP IN LOWER 2KBYTE
	
		clrf	PORTA		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTB		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTC		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTD		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTE		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
	
		bcf		STATUS, RP1	; FOR BANK 1 RP1=0 RP0=1.
		bsf		STATUS, RP0	; SELECT BANK 1.
	
		clrf    PORTB		;LATCH ALL LOW.
		movlw   B'11110000'	;RB4-RB7 INPUTS. 
		movwf	TRISB	
			
		movlw	B'00000000'	; RD4-7 INPUTS.
		movwf	TRISD
		
		bcf		OPTION_REG, NOT_RBPU	
						;ENABLE WEAK PULL-UPS.
						;ON PORT B.
	
		bcf		STATUS, RP0	; SELECT BANK 0.
		
		bcf		INTCON, INTF; CLEAR INTERRUPT FLAG.
		bsf		INTCON, RBIE; ENABLE INTERRUPT ON RB4-7 CHANGE.
		bsf		INTCON, GIE	; ENABLE INTERRUPTS. 
		
		return

;=============================================================================
;FLASH ALL FOUR LEDS.
;=============================================================================
FLASH4		
		movlw	B'0001111'	
		movwf	PORTD
		call	WAIT_15MS
		movlw	B'00000000'
		movwf	PORTD
		
		return

;=============================================================================	
; DISPLAY THE OPENING MESSAGE.
;===============================================================================  
OPEN_MESG
call WAIT_64MS

		movlw	'F'			;ENTER "F".
		call	LCD_DISP_CHAR

		movlw	'R'			;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	'E'			;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'Q'			;ENTER "Q".
		call	LCD_DISP_CHAR

		movlw	' '			;ENTER " ".
		call	LCD_DISP_CHAR
		
		movlw	'G'			;ENTER "G".
		call	LCD_DISP_CHAR

		movlw	'E'			;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'N'			;ENTER "N".
		call	LCD_DISP_CHAR

		movlw	'E'			;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'R'			;ENTER "R".
		call	LCD_DISP_CHAR
		
		movlw	'A'			;ENTER "A".
		call	LCD_DISP_CHAR

		movlw	'T'			;ENTER "T".
		call	LCD_DISP_CHAR

		movlw	'O'			;ENTER "O".
		call	LCD_DISP_CHAR
		
		movlw	'R'			;ENTER "R".
		call	LCD_DISP_CHAR
		
		movlw	'.'			;ENTER ".".
		call	LCD_DISP_CHAR


		call 	WAIT_500MS
	
		call 	LCD_CLEAR
		
		call 	WAIT_15MS

		return


;=============================================================================
;SHIFT KEYS UP TO NEXT SCALE.
;=============================================================================
SHIFT_KEYS
		;clrf 	FSR
		movlw h'09'
		movf	LAST_CHAR	;THIS IS THE FIRST CHAR SO CHECK IF ITS >"0".
			movwf	FSR			;FSR IS AT THE FIRST ADDRESS ENTERED.
		
		
		
		movf	FCHAR_MAX, w
		subwf	FCHAR_STRT, w
		movwf	FSR			;FSR IS AT THE FIRST ADDRESS ENTERED.
		
		movf	FCHAR_MAX,W	;FCHAR_MAX IS OUR RUNNING COUNT.
		movwf	FCHAR_CNT	;WE CAN DEC THIS VAR WITHOUT SCREWING UP CHAR CNT.
		btfss	STATUS,Z	;CHECK FCHAR_CNT > 0?
		goto 	EXCEED_MAX_FREQ? 	;WE HAVE MORE THAN ONE CHARACTER IN OUR FREQ..
							;ELSE ITS THE FIRST CHAR...		
		movf	LAST_CHAR,W	;THIS IS THE FIRST CHAR SO CHECK IF ITS >"0".
		btfss	STATUS,C	;CHECK FCHAR_CNT = 0?
		goto	ZERO_ERROR	;DROP OUT. CANT HAVE "0" AS FIRST CHAR. 
		incf	FCHAR_MAX	;YES ITS > "0" SO WE CAN NOW INC OUR COUNTERS.
		incf	FCHAR_CNT
		movwf   INDF		;Happens here INDF is not applied to address 027	




		movlw	H'05'
		movwf	LAST_CHAR		
				
		RETURN
		
EXCEED_MAX_FREQ?
		movf	FCHAR_CNT,W	;NOW CHECK IF WE ARE AT X0Mhz AND CHECK X<6.
		sublw	D'07'
		btfsc	STATUS,C	;CHECK FCHAR_CNT > 7?
		goto	SHIFT_CHAR	;YES ITS LOWER THAN "7" SO DONT CHECK 5MEG.
		movf	LAST_CHAR,W
		sublw	D'06'
		btfsc	STATUS,C	;CHECK LAST_CHAR > 6?
		goto	SHIFT_CHAR	;YES ITS LOWER THAN "6" OR A POSSIBLE 60MEG.
		goto	FREQ_TO_BIG	;DISPLAY THE "FREQ TO BIG" LABEL.
				
SHIFT_CHAR
		incf	FSR			;FSR IS ONE GREATER THAN OUR LAST CHAR... 	
		movf	INDF,W
		decf	FSR			;INCREMENT TO THE NEW LOCATION (ONE UP).	
		movwf	INDF		;DEPOSIT THE LOCATION VALUE THERE.
		incf	FSR			;SHIFT DOWN TO PREVIOUS LOCATION.		
		decf	FCHAR_CNT	;WE DIDNT "INC" THIS SO "DEC_ING" IT GOES BACK ONE.
		movf	FCHAR_CNT,F	;CHECK IF ITS BACK TO ZERO?
		btfsc	STATUS,Z
		goto	LOAD_LAST_CHAR ;YES. ITS OUR LAST CHAR SO SAVE IT TO LOC="0"
		goto	SHIFT_CHAR	;NO. BACK TO THE TOP TO MOVE THE NEXT CHARACTER.

LOAD_LAST_CHAR
		incf	FCHAR_MAX	;INC OUR CHARACTER COUNT.
		movf	LAST_CHAR,W	;SAVE THE KEY TO THE LOCATION AND RETURN.
		movwf   INDF	

DISP_CHARS
		movlw	H'08'		;POSITION CURSOR AND SWAPING THE LCD CHAR DIRECTION.
		call	LCD_LINE1_W
		call	LCD_DEC_CUR

		Movf	FCHAR_MAX,W
		Movwf	FCHAR_CNT	;SET NUMBER OF CHARACTERS TO HANDLE.
		
LOAD_CHAR	
		Movf	FCHAR_CNT, W
		btfsc	STATUS, Z 	;CHECK IF THIS IS THE LAST CHARACTER HANDLED?
		goto	DISP_CLEAN
		goto	CHAR_TO_LCD ;NO, GET CHARACTER AND SEND IT TO LCD.
				
CHAR_TO_LCD	
		movf	INDF, W		;GET CHARACTER FROM POINTER POSITION.
		call	LCD_PUT_ASC ;TO LCD.
		decf	FCHAR_CNT	;DECREMENT CHARACTER COUNTER.
		decf	FSR			;INCREMENT POINTER.
		goto	LOAD_CHAR	
			
DISP_CLEAN		
		call	LCD_INC_CUR		
		call 	WAIT_64MS
		
		
		
		

		
		return



;=============================================================================
;SAVE KEY - old.
;=============================================================================
SAVE_KEY
		clrf 	FSR
		movf	FCHAR_MAX,w
		
		subwf	FCHAR_STRT,w
		movwf	FSR
		movf	LAST_KEY,W
		movwf   INDF	
	
		incf	FCHAR_MAX
		
			clrf 	FSR
		
	
		return
;=============================================================================
;DISPLAY THE FREQUENCY LABEL.
;=============================================================================
FREQ_LABEL
		clrf 	FSR
		movlw	H'08'
		call	LCD_LINE1_W
		call	LCD_DEC_CUR

		Movf	FCHAR_MAX,w
		Movwf	FCHAR_CNT	;SET NUMBER OF CHARACTERS TO HANDLE.
		
		
		movf	FCHAR_STRT,w		
		Movwf	FSR			;MOVE THAT NUMBER TO FSR ALSO.
		bcf		STATUS,Z
		
CHARLINE0	
		Movf	FCHAR_CNT, W
		btfss	STATUS, Z 	;LAST CHARACTER HANDLED?
		Goto	CHAR_TO_LCD1 ;NO, GET CHARACTER AND SEND IT TO LCD.
		goto	DISP_CLEAN1
		
CHAR_TO_LCD1	
		movf	INDF, W		;GET CHARACTER FROM POINTER POSITION.
		call	LCD_PUT_ASC ;TO LCD.
		decf	FCHAR_CNT	;DECREMENT CHARACTER COUNTER.
		decf	FSR			;INCREMENT POINTER.
		goto	CHARLINE0	
			
DISP_CLEAN1		
		call	LCD_INC_CUR		
		call 	WAIT_64MS
				
		return

;=============================================================================
; DISPLAY THE HZ LABEL.
;=============================================================================
HZ_LABEL
		movlw	H'09'
		call	LCD_LINE1_W

		movlw	'H'		;ENTER "H".
		call	LCD_DISP_CHAR

		movlw	'z'		;ENTER "z".
		call	LCD_DISP_CHAR

		movlw	'-'		;ENTER "-".
		call	LCD_DISP_CHAR

		movlw	'F'		;ENTER "F".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	'E'		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'Q'		;ENTER "Q".
		call	LCD_DISP_CHAR
	
		;call 	WAIT_128MS
	
		return



;=============================================================================
; DISPLAY THE ERROR FIRST CHAR IS EQUAL TO ZERO.
;=============================================================================
ZERO_ERROR
		call	LCD_CLEAR
		call	LCD_BUSY
		call	LCD_LINE1	

		movlw	'E'		;ENTER "H".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "z".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "-".
		call	LCD_DISP_CHAR

		movlw	'O'		;ENTER "F".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	' '		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'-'		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'>'		;ENTER "E".
		call	LCD_DISP_CHAR
		
		movlw	' '		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'F'		;ENTER "Q".
		call	LCD_DISP_CHAR
	
		movlw	'I'		;ENTER "H".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "z".
		call	LCD_DISP_CHAR

		movlw	'S'		;ENTER "-".
		call	LCD_DISP_CHAR
		
		movlw	'T'		;ENTER "-".
		call	LCD_DISP_CHAR

		call	LCD_BUSY
		call	LCD_LINE2	
		
		movlw	'C'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	'H'		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'A'		;ENTER "Q".
		call	LCD_DISP_CHAR
		
		movlw	'R'		;ENTER "Q".
		call	LCD_DISP_CHAR

		movlw	' '		;ENTER "Q".
		call	LCD_DISP_CHAR
		
		movlw	'M'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	'U'		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'S'		;ENTER "Q".
		call	LCD_DISP_CHAR
		
		movlw	'T'		;ENTER "H".
		call	LCD_DISP_CHAR

		movlw	' '		;ENTER "z".
		call	LCD_DISP_CHAR

		movlw	'N'		;ENTER "-".
		call	LCD_DISP_CHAR

		movlw	'O'		;ENTER "F".
		call	LCD_DISP_CHAR

		movlw	'T'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	' '		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'='		;ENTER "Q".
		call	LCD_DISP_CHAR
	
		movlw	'0'		;ENTER "z".
		call	LCD_DISP_CHAR
				
		call 	WAIT_A_SEC
	
		goto	HZ_LABEL
		return

;=============================================================================
; DISPLAY THE LABEL FOR FREQUENCY GREATER THAN 50 MEG.
;=============================================================================
FREQ_TO_BIG
		call	LCD_CLEAR
		call	LCD_BUSY
		call	LCD_LINE1	

		movlw	'E'		;ENTER "H".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "z".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "-".
		call	LCD_DISP_CHAR

		movlw	'O'		;ENTER "F".
		call	LCD_DISP_CHAR

		movlw	'R'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	' '		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'-'		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'>'		;ENTER "E".
		call	LCD_DISP_CHAR
		
		movlw	' '		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'F'		;ENTER "Q".
		call	LCD_DISP_CHAR
	
		movlw	'R'		;ENTER "H".
		call	LCD_DISP_CHAR

		movlw	'E'		;ENTER "z".
		call	LCD_DISP_CHAR

		movlw	'Q'		;ENTER "-".
		call	LCD_DISP_CHAR
		

		call	LCD_BUSY
		call	LCD_LINE2	
		
		movlw	'>'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	' '		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	'6'		;ENTER "Q".
		call	LCD_DISP_CHAR
		
		movlw	'0'		;ENTER "Q".
		call	LCD_DISP_CHAR

		movlw	'M'		;ENTER "Q".
		call	LCD_DISP_CHAR
		
		movlw	'E'		;ENTER "R".
		call	LCD_DISP_CHAR

		movlw	'G'		;ENTER "E".
		call	LCD_DISP_CHAR

		movlw	' '		;ENTER "Q".
		call	LCD_DISP_CHAR
		
		movlw	'H'		;ENTER "H".
		call	LCD_DISP_CHAR

		movlw	'z'		;ENTER "z".
		call	LCD_DISP_CHAR

		movlw	'.'		;ENTER "-".
		call	LCD_DISP_CHAR

			
		call 	WAIT_A_SEC
		
		movlw	H'00'	;RESET THE CHARACTER COUNT...
		movwf	FCHAR_MAX
	
		goto	HZ_LABEL
		return




	
;===========================================================================
;DELAY ROUTINES.
;***************
;
; DELAY_TIME	= ((DELAY_value * 3) + 4) * Cycle_time
; DELAY_VALUE	= (Delay_time - (4 * Cycle_time)) / (3 * Cycle_time)
;
; i.e. (@ 4MHz crystal)
; Delay_time	= ((32 * 3) + 4) * 1uSec
;		= 100uSec
; DELAY_value	= (500uSec - 4) / 3
;		= 165.33
;		= 165
;===========================================================================
WAIT_A_SEC	
		call WAIT_500MS
		call WAIT_500MS
		
WAIT_500MS	
		call WAIT_256MS
		call WAIT_256MS	

WAIT_256MS	
		call WAIT_128MS
		call WAIT_128MS
		
		
WAIT_128MS	
		movlw	D'240'		; 128 msec
		movwf	X_DLY_VAL
LOOP_128MS	
		call	DELAY		;WAIT 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_128MS
		return
		
WAIT_64MS	
		movlw	D'120'		; 64 msec
		movwf	X_DLY_VAL	;
LOOP_64MS	
		call	DELAY		;WAIT 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_64MS	
		return
		
WAIT_32MS	
		movlw	D'60'		; 30 msec
		movwF	X_DLY_VAL
LOOP_32MS	
		call	DELAY		; WAIT 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_32MS
		return


WAIT_15MS	
		movlw	D'30'		; 15 msec
		movwf	X_DLY_VAL
LOOP_15MS	
		call	DELAY		;wait 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_15MS
		return

;DELAY ROUTINE = 500uSec OR 1/2 A MILISECOND.
DELAY		
		movlw	H'FF'		; +1		1 CYCLE
		movwF	DELAY_VAL	; +2		1 CYCLE
		
DELAY_LOOP	
		decfsz	DELAY_VAL, F; step 1	1 CYCLE
		goto	DELAY_LOOP	; step 2	2 CYCLE
		return			; +3		2 CYCLE
		
		end
